@@ -14,6 +14,7 @@ try:
     from igel.data import models_dict, metrics_dict
     from igel.preprocessing import update_dataset_props
     from igel.preprocessing import handle_missing_values, encode, normalize
+    from igel.hyperparams import hyperparameter_search
 except ImportError:
     from utils import read_yaml, create_yaml, extract_params, _reshape, read_json
     from data import evaluate_model
@@ -21,6 +22,7 @@ except ImportError:
     from data import models_dict, metrics_dict
     from preprocessing import update_dataset_props
     from preprocessing import handle_missing_values, encode, normalize
+    from hyperparams import hyperparameter_search
 
 from sklearn.model_selection import train_test_split, cross_validate
 from sklearn.multioutput import MultiOutputClassifier, MultiOutputRegressor
@@ -284,23 +286,23 @@ class Igel(object):
         return self._process_data(target='predict')
 
     def get_evaluation(self, model, x_test, y_true, y_pred, **kwargs):
-        res = None
         try:
             res = evaluate_model(model_type=self.model_type,
-                                  model=model,
-                                  x_test=x_test,
-                                  y_pred=y_pred,
-                                  y_true=y_true,
-                                  get_score_only=False,
-                                  **kwargs)
+                                 model=model,
+                                 x_test=x_test,
+                                 y_pred=y_pred,
+                                 y_true=y_true,
+                                 get_score_only=False,
+                                 **kwargs)
         except Exception as e:
+            logger.debug(e)
             res = evaluate_model(model_type=self.model_type,
-                                  model=model,
-                                  x_test=x_test,
-                                  y_pred=y_pred,
-                                  y_true=y_true,
-                                  get_score_only=True,
-                                  **kwargs)
+                                 model=model,
+                                 x_test=x_test,
+                                 y_pred=y_pred,
+                                 y_true=y_true,
+                                 get_score_only=True,
+                                 **kwargs)
         return res
 
     def fit(self, **kwargs):
@@ -315,6 +317,8 @@ class Igel(object):
         cv_results = None
         eval_results = None
         cv_params = None
+        hp_search_results = {}
+
         if self.model_type == 'clustering':
             x_train = self._prepare_clustering_data()
         else:
@@ -334,11 +338,33 @@ class Igel(object):
             if not cv_params:
                 logger.info(f"cross validation is not provided")
             else:
+                # perform cross validation
+                logger.info("performing cross validation ...")
                 cv_results = cross_validate(estimator=self.model,
                                             X=x_train,
-                                            y=y_train, **cv_params)
+                                            y=y_train,
+                                            **cv_params)
+            hyperparams_props = self.model_props.get('hyperparameter_search', None)
+            if hyperparams_props:
+                method = hyperparams_props.get('method', None)
+                grid_params = hyperparams_props.get('parameter_grid', None)
+                hp_args = hyperparams_props.get('arguments', None)
+                logger.info(f"Performing hyperparameter search using -> {method}")
+                logger.info(f"Grid parameters entered by the user: {grid_params}")
+                logger.info(f"Additional hyperparameter arguments: {hp_args}")
+                best_estimator, best_params, best_score = hyperparameter_search(model=self.model,
+                                                                                method=method,
+                                                                                params=grid_params,
+                                                                                x_train=x_train,
+                                                                                y_train=y_train,
+                                                                                **hp_args)
+                hp_search_results['best_params'] = best_params
+                hp_search_results['best_score'] = best_score
+                self.model = best_estimator
+
             self.model.fit(x_train, y_train)
-        else:
+
+        else:   # if the model type is clustering
             self.model.fit(x_train)
 
         saved = self._save_model(self.model)
@@ -377,7 +403,8 @@ class Igel(object):
             "results_path": str(self.results_path),
             "model_path": str(self.default_model_path),
             "target": None if self.model_type == 'clustering' else self.target,
-            "results_on_test_data": eval_results
+            "results_on_test_data": eval_results,
+            "hyperparameter_search_results": hp_search_results
 
         }
         if self.model_type == 'clustering':
@@ -482,17 +509,3 @@ class Igel(object):
         else:
             logger.warning(f"something went wrong while initializing a default file")
 
-
-if __name__ == '__main__':
-    mock_fit_params = {'data_path': '/home/nidhal/projects/igel/examples/multioutput-example/linnerud.csv',
-                       'yaml_path': '/home/nidhal/projects/igel/examples/multioutput-example/multioutput.yaml',
-                       #/home/nidhal/projects/igel/examples/indian-diabetes-example/random-forest.yaml
-                       'cmd': 'fit'}
-    mock_eval_params = {'data_path': '/home/nidhal/projects/igel/examples/data/indian-diabetes/eval-indians-diabetes.csv',
-                        'cmd': 'evaluate'}
-    mock_pred_params = {'data_path': '/home/nidhal/projects/igel/examples/data/indian-diabetes/test-indians-diabetes.csv',
-                        'cmd': 'predict'}
-
-    Igel(**mock_fit_params)
-    # Igel(**mock_eval_params)
-    # Igel(**mock_pred_params)
